@@ -26,33 +26,36 @@ keyword_type_value = ('int', 'short', 'bool', 'string', 'void', 'char', 'ulong',
 class File:
     def __init__(self, path):
         self.path = path
-        file = open(path, encoding="utf-8")
+        file = open(path, encoding="utf-8", errors='ignore')
         self.all_tokens = tokenize(file.read())
 
 
+# PascalCase
+def validate_pascal_case(token):
+    if token.correct_token_value.isupper():
+        token.correct_token_value = token.correct_token_value.casefold()
+    index = 1
+    while index < len(token.correct_token_value):
+        if token.correct_token_value[index] == '_' and index + 1 < len(token.correct_token_value):
+            if token.correct_token_value[index] == '_' and token.correct_token_value[index + 1] == '_':
+                token.correct_token_value = token.correct_token_value.replace('_', '', 1)
+                index -= 1
+            else:
+                token.correct_token_value = token.correct_token_value[:index] + \
+                                            token.correct_token_value[index + 1].upper() + \
+                                            token.correct_token_value[index + 2:]
+        index += 1
+    token.correct_token_value = token.correct_token_value.replace('_', '')
+    token.correct_token_value = token.correct_token_value[0].upper() + token.correct_token_value[1:]
+
+
 class StaticAnalyzer:
 
     def __init__(self, files):
         self.files = files
 
-    # PascalCase
-    def validate_pascal_case(self, token):
-        index = 1
-        while index < len(token.correct_token_value):
-            if token.correct_token_value[index] == '_' and index + 1 < len(token.correct_token_value):
-                if token.correct_token_value[index] == '_' and token.correct_token_value[index + 1] == '_':
-                    token.correct_token_value = token.correct_token_value.replace('_', '', 1)
-                    index -= 1
-                else:
-                    token.correct_token_value = token.correct_token_value[:index] + \
-                                                token.correct_token_value[index + 1].upper() + \
-                                                token.correct_token_value[index + 2:]
-            index += 1
-        token.correct_token_value = token.correct_token_value.replace('_', '')
-        token.correct_token_value = token.correct_token_value[0].upper() + token.correct_token_value[1:]
-
     def validate_interface(self, token):
-        self.validate_pascal_case(token)
+        validate_pascal_case(token)
         if token.correct_token_value[0] != 'I':
             token.correct_token_value = 'I' + token.correct_token_value
         else:
@@ -61,7 +64,7 @@ class StaticAnalyzer:
                                         token.correct_token_value[2:]
 
     def validate_camel_case(self, token):
-        self.validate_pascal_case(token)
+        validate_pascal_case(token)
         token.correct_token_value = token.correct_token_value[0].lower() + token.correct_token_value[1:]
 
     def rename_all(self, token_for_rename):
@@ -96,17 +99,17 @@ class StaticAnalyzer:
                 next_significant_token = file.all_tokens[find_next_significant_token_index(file, index)]
                 if len(stack_influential_tokens) > 0 and stack_influential_tokens[-1].token_value in \
                         ['namespace', 'enum']:
-                    self.validate_pascal_case(current_token)  # namespaces, enums
+                    validate_pascal_case(current_token)  # namespaces, enums
                     self.rename_all(current_token)
                 elif len(stack_influential_tokens) > 1 and previous_significant_token.token_value == 'class':
-                    self.validate_pascal_case(current_token)  # classes
+                    validate_pascal_case(current_token)  # classes
                     self.rename_all(current_token)
                 elif len(stack_influential_tokens) > 0 and stack_influential_tokens[-1].token_value == 'interface':
                     self.validate_interface(current_token)  # interfaces
                     self.rename_all(current_token)
                 elif len(stack_influential_tokens) > 1 and stack_influential_tokens[-1].token_value == '{' and \
                         stack_influential_tokens[-2] == 'enum':
-                    self.validate_pascal_case(current_token)  # enum values
+                    validate_pascal_case(current_token)  # enum values
                     self.rename_all(current_token)
                 elif len(stack_influential_tokens) > 1 and stack_influential_tokens[-1].token_value == '{' and \
                         stack_influential_tokens[-2].token_value in ['class', 'interface'] and \
@@ -114,10 +117,10 @@ class StaticAnalyzer:
                          previous_significant_token.token_type == TokenType.NUMBER_OR_IDENTIFIERS or
                          previous_significant_token.token_value in keyword_type_value):
                     if was_const:
-                        self.validate_pascal_case(current_token)  # constants
+                        validate_pascal_case(current_token)  # constants
                         self.rename_all(current_token)
                     elif next_significant_token.token_value in ['(', '{']:
-                        self.validate_pascal_case(current_token)  # methods and properties
+                        validate_pascal_case(current_token)  # methods and properties
                         self.rename_all(current_token)
                     elif next_significant_token.token_value in [';', '=', ',', ')']:
                         self.validate_camel_case(current_token)  # object references
@@ -167,6 +170,7 @@ class StaticAnalyzer:
             return result
 
     def get_valid_documentation(self, file, index):
+        name_index = index
         indent = 0
         row = 0
         document_comments = []
@@ -176,17 +180,23 @@ class StaticAnalyzer:
                 row = file.all_tokens[index].row
                 document_comments.insert(0, file.all_tokens[index].correct_token_value)
                 file.all_tokens.pop(index)
+                name_index -= 1
                 index -= 1
                 while file.all_tokens[index].correct_token_value in [' ', '\t']:
                     file.all_tokens.pop(index)
+                    name_index -= 1
                     index -= 1
                 if file.all_tokens[index].correct_token_value == '\n':
                     file.all_tokens.pop(index)
+                    name_index -= 1
                     index -= 1
                 index += 1
 
             index -= 1
 
+        while file.all_tokens[index].token_value != '\n':
+            index += 1
+
         if indent == 0:
             indent = file.all_tokens[find_next_significant_token_index(file, index)].column - 1
         document_comments_text = ''
@@ -196,9 +206,9 @@ class StaticAnalyzer:
             document_comments_text += line[3:]
 
         correct_documentation = self.validate_doc_text(document_comments_text)
-        return correct_documentation, indent, row, index, document_comments
+        return correct_documentation, indent, row, index, document_comments, name_index
 
-    def add_documented_comment(self, file, index, correct_documentation, indent, row, comments_before):
+    def add_documented_comment(self, file, index, correct_documentation, indent, row, comments_before, name_index):
         ind = 0
         document_comments = correct_documentation.split('\n')
         for doc_line in document_comments:
@@ -206,25 +216,30 @@ class StaticAnalyzer:
             for i in range(indent):
                 file.all_tokens.insert(index, Token(TokenType.WHITE_SPACE, ' ', None, None))
                 index += 1
+                name_index += 1
 
             curr_index = 0
             while curr_index < len(doc_line) and doc_line[curr_index].isspace():
                 curr_index += 1
             comment_str = '/// ' + doc_line[curr_index:]
             file.all_tokens.insert(index, Token(TokenType.COMMENT, "", row, indent + 1))
+            name_index += 1
             if ind < len(comments_before):
                 file.all_tokens[index].token_value = comments_before[ind]
             file.all_tokens[index].correct_token_value = comment_str
             index += 1
             file.all_tokens.insert(index, Token(TokenType.WHITE_SPACE, '\n', None, None))
+            name_index += 1
             row += 1
             ind += 1
+        return name_index
 
     # class, interface, enum
     def validate_doc_type_comment(self, name_index, word_type, file):
         name = file.all_tokens[name_index].correct_token_value
         index = name_index
-        correct_documentation, indent, row, index, document_comments = self.get_valid_documentation(file, index)
+        correct_documentation, indent, row, index, document_comments, name_index \
+            = self.get_valid_documentation(file, index)
 
         if correct_documentation.find('<summary>') == -1:
             correct_documentation = '<summary>\n' + name + \
@@ -234,12 +249,13 @@ class StaticAnalyzer:
         while file.all_tokens[index].token_value != '\n':
             index -= 1
 
-        self.add_documented_comment(file, index, correct_documentation, indent, row, document_comments)
-
-        while True:
-            if file.all_tokens[index].correct_token_value == name:
-                return index
-            index += 1
+        name_index = self.add_documented_comment(file, index, correct_documentation, indent, row, document_comments,
+                                                 name_index)
+        return name_index
+        # while True:
+        #     if file.all_tokens[index].correct_token_value == name and file.all_tokens[index + 1].token_value != '<':
+        #         return index
+        #     index += 1
 
     def validate_doc_method_comment(self, name_index, file):
         name = file.all_tokens[name_index].correct_token_value
@@ -250,11 +266,12 @@ class StaticAnalyzer:
         while file.all_tokens[param_index].token_value != '(':
             param_index += 1
         while file.all_tokens[param_index].token_value != ')':
-            if file.all_tokens[param_index + 1].token_value in [',', ')']:
+            if file.all_tokens[find_next_significant_token_index(file, param_index)].token_value in [',', ')', '='] \
+                    and file.all_tokens[param_index].token_type == TokenType.NUMBER_OR_IDENTIFIERS:
                 params.append(file.all_tokens[param_index].correct_token_value)
             param_index += 1
 
-        correct_documentation, indent, row, index, documents_comments = self.get_valid_documentation(file, index)
+        correct_documentation, indent, row, index, documents_comments, _ = self.get_valid_documentation(file, index)
 
         if correct_documentation.find('<summary>') == -1:
             correct_documentation = '<summary>\n' + name + \
@@ -263,8 +280,8 @@ class StaticAnalyzer:
         corr_index = 0
         pattern = '<param name="'
         while corr_index + len(pattern) < len(correct_documentation):
-            if correct_documentation[corr_index:corr_index+len(pattern)] == pattern:
-                corr_index = corr_index+len(pattern)
+            if correct_documentation[corr_index:corr_index + len(pattern)] == pattern:
+                corr_index = corr_index + len(pattern)
                 start_of_name = corr_index
                 while correct_documentation[corr_index] != '"':
                     corr_index += 1
@@ -281,7 +298,7 @@ class StaticAnalyzer:
         while file.all_tokens[index].token_value != '\n':
             index -= 1
 
-        self.add_documented_comment(file, index, correct_documentation, indent, row, documents_comments)
+        self.add_documented_comment(file, index, correct_documentation, indent, row, documents_comments, name_index)
 
     def validate_documentation(self, file):
         stack_influential_tokens = []
@@ -293,6 +310,7 @@ class StaticAnalyzer:
             elif current_token.token_value == ']':
                 stack_influential_tokens.pop()
             elif current_token.token_value == '}':
+                # print(current_token, stack_influential_tokens)
                 stack_influential_tokens.pop()
                 if len(stack_influential_tokens) > 0 and stack_influential_tokens[-1].token_value in \
                         ['class', 'enum', 'namespace', 'interface']:
@@ -301,10 +319,12 @@ class StaticAnalyzer:
             if current_token.token_type == TokenType.NUMBER_OR_IDENTIFIERS:
                 previous_significant_token = file.all_tokens[find_previous_significant_token_index(file, index)]
                 next_significant_token = file.all_tokens[find_next_significant_token_index(file, index)]
-                if len(stack_influential_tokens) > 0 and stack_influential_tokens[-1].token_value in ['enum',
-                                                                                                      'class',
-                                                                                                      'interface']:
-                    self.validate_doc_type_comment(index, stack_influential_tokens[-1].token_value, file)
+                if len(stack_influential_tokens) > 0 and previous_significant_token.token_value == \
+                        stack_influential_tokens[-1].token_value \
+                        and stack_influential_tokens[-1].token_value in ['enum',
+                                                                         'class',
+                                                                         'interface']:
+                    index = self.validate_doc_type_comment(index, stack_influential_tokens[-1].token_value, file)
                 elif len(stack_influential_tokens) > 1 and stack_influential_tokens[-1].token_value == '{' and \
                         stack_influential_tokens[-2].token_value in ['class', 'interface', 'enum'] and \
                         (previous_significant_token.token_value in ['>', ']'] or
@@ -313,12 +333,15 @@ class StaticAnalyzer:
                     if next_significant_token.token_value == '(':
                         self.validate_doc_method_comment(index, file)  # methods
                     if next_significant_token.token_value == '{':
-                        self.validate_doc_type_comment(index, 'property', file)  # properties
+                        index = self.validate_doc_type_comment(index, 'property', file)  # properties
+                        # if previous_significant_token.token_value == current_token.token_value:
+                        #     index += 2
                     elif next_significant_token.token_value in [';', '=']:
                         self.validate_doc_type_comment(index, 'object reference', file)  # object references
             index += 1
 
     def analyze(self):
         for file in self.files:
+            print(file.path)
             self.validate_names(file)
             self.validate_documentation(file)
